<?php

namespace Drupal\tellstick;

use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\tellstick\Entity\TellstickSensor;

/**
 * Class Telsstick service.
 *
 *  Handles search requests for Tellstick Sensors.
 */
class TellstickService implements TellstickServiceInterface {

  /**
   * The entity query manager injected into the service.
   *
   * @var \Drupal\Core\Entity\Query\QueryFactory
   */
  private $entityQueryManager;

  /**
   * The current user injected into the service.
   *
   * @var Drupal\Core\Session\AccountInterface
   */
  private $currentUser;

  /**
   * The type Entity Type Manager injected into the service.
   *
   * @var Drupal\Core\Entity\EntityTypeManagerInterface
   */
  private $entityTypeManager;

  /**
   * The Tellstick Live Sensor Connector.
   *
   * @var Drupal\tellstick\TellstickLiveService
   */
  private $tellstickLiveService;

  /**
   * Constructor.
   *
   * @param Drupal\Core\Session\AccountInterface $current_user
   *   The current user.
   * @param Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity manager.
   * @param Drupal\tellstick\TellstickLiveService $tellstick_live_service
   *   The entity manager.
   */
  public function __construct(AccountInterface $current_user,
                              EntityTypeManagerInterface $entity_type_manager,
                              TellstickLiveService $tellstick_live_service) {
    $this->currentUser = $current_user;
    $this->entityTypeManager = $entity_type_manager;
    $this->tellstickLiveService = $tellstick_live_service;
  }

  /**
   * Get all Tellstick Sensors.
   *
   * @return Drupal\tellstick\Entity\TellstickSensor[]
   *   An array of found Flag For Lists.
   */
  public function getAllTellstickSensors() {
    $query = $this->entityTypeManager
                  ->getStorage('tellstick_sensor')
                  ->getQuery();
    $ids = $query->execute();

    return $ids;
  }

  /**
   * Get a Tellsick Sensor by Id.
   *
   * @param string $sensor_id
   *   The Sensor Id of the Tellstick Sensor to load.
   *
   * @return Drupal\tellstick\Entity\TellstickSensor
   *   The loaded Telsstick Sensor.
   */
  public function getTellstickSensorById($sensor_id) {
    // Try first the Sensor Entity.
    $sensor = $this->entityTypeManager
      ->getStorage('tellstick_sensor')
      ->load($sensor_id);
    if (empty($sensor)) {
      // Try loading from the Telldus Live instead.
      $live_sensor = $this->tellstickLiveService
        ->getTellstickLiveSensorInfo($sensor_id);
      if (!empty($live_sensor)) {

        // Save new data into TellstickSensor entity.
        $name = $live_sensor['name'];
        $model = $live_sensor['model'];
        $clientName = $live_sensor['clientName'];
        $ignored = $live_sensor['ignored'];
        $created = $live_sensor['lastUpdated'];
        $data = ['Battery' => (string) $live_sensor['battery']];
        $tellstick_sensor = TellstickSensor::create([
          'id' => $sensor_id,
          'sensor' => (string) $name,
          'sensorType' => (string) $model,
          'ignored' => (string) $ignored,
          'place' => (string) $clientName,
          'created' => (string) $created,
          'weight' => 0,
          'data' => serialize($data),
        ]);
        $tellstick_sensor->save();
        // Now when it is loaded and stored, recursivly call
        // ourself again.
        $sensor = $this->getTellstickSensorById($sensor_id);
      }
    }
    return $sensor;
  }

  /**
   * Loads Tellstick Sensors by ids.
   *
   * @param int[] $ids
   *   The Ids of the Tellstick Sensors to load.
   *
   * @return Drupal\tellstick\Entity\TellstickSensor
   *   The loaded Tellstick Sensors.
   */
  protected function getTellstickSensorByIds(array $ids) {
    return $this->entityTypeManager->getStorage('tellstick_sensor')->loadMultiple($ids);
  }

  /**
   * Get a Tellstick Data Item.
   *
   * @param int $id
   *   The Tellstick Data Item ids to load.
   *
   * @return Drupal\tellstick\Entity\TellstickData[]
   *   A Tellstick Data Item.
   */
  public function getTellstickDataById($id) {
    return $this->entityTypeManager->getStorage('tellstick_data')->load($id);
  }

  /**
   * Get Tellstick Data Items.
   *
   * @param int[] $ids
   *   The Tellstick Data Item ids to load.
   *
   * @return Drupal\tellstick\Entity\TellstickData[]
   *   An array of Tellstick Data Items.
   */
  public function getTellstickDataByIds(array $ids) {
    return $this->entityTypeManager->getStorage('tellstick_data')->loadMultiple($ids);
  }

}
